#!/bin/sh

# Copyright 2014, 2016, 2019, 2020, 2021 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

. "$_EXEC/pdiread.sh"
. "$_EXEC/session_lock.sh"
. "$_EXEC/cgilite/storage.sh"

unset filter order card action newfield
unset cardfile attfile tempfile
unset vcf field cnt delete_key

filter="$(REF f)"
order="$(REF o)"

card="$(POST card |PATH)"; card="${card##*/}"
cardfile="$_DATA/vcard/${card}"
attfile="$_DATA/mappings/attendance"

action="$(POST action)"
newfield="$(POST newfield |grep -m 1 -xE '[A-Z][A-Z0-9-]*')"

if printf '%s\n' "$action" |grep -qxE 'addfield [A-Z][A-Z0-9]*'; then
  newfield="${action##* }"
  action=addfield
fi

if ! tempfile=$(CHECK_SLOCK "$cardfile"); then
  SET_COOKIE 0 message="NO VALID FILE LOCK"
  REDIRECT "/cards/?o=${order}&f=${filter}&e=${card}"
  exit 0
elif [ "$(POST tid)" != "$(transid "$tempfile")" ]; then
  SET_COOKIE 0 message="INVALID TRANSACTION ID"
  REDIRECT "/cards/?o=${order}&f=${filter}&e=${card}"
  exit 0
fi

vcf_escape(){
  for each in "$@"; do
    printf %s\\n "$each" \
    | sed -E ':X;$!{N;bX}; s;\r\n;\n;g; s;([;,\\]);\\\1;g; s;\n;\\n;g;'
  done \
  | sed -E ':X;$!{N;bX}; s;\n;\;;g'
}

# [ "${_POST[hi_select]}" = "list" ] || _POST[hi_company]="${_POST[hi_other]}"
# [ -n "${_POST[hi_company]}${_POST[hi_number]}${_POST[hi_status]}" ] \
# && _POST[X-HEALTH-INSURANCE]="$(vcf_escape "${_POST[hi_company]}" "${_POST[hi_number]}" "${_POST[hi_status]}")"

vcf="$(pdi_load "$cardfile")"

vcf="$(pdi_update_value "$vcf" N 1 "$(vcf_escape "$(POST 1N)" "$(POST 2N)" "$(POST 3N)" "$(POST 4N)" "$(POST 5N)")")"
vcf="$(printf '%s\n' "$vcf" |sed -E "/^CATEGORIES;[^:]*:.*$/d")"

for field in $(POST_KEYS |grep -xE '[A-Z][A-Z0-9-]*'); do
  for cnt in $(seq 1 $(POST_COUNT "$field")); do
    case "$field" in
      # (TEL)
      #   printf '%s;TYPE=%s:%s\r\n' "${field}" "${_POST[phonetype${key#TEL}]}" "$(vcf_escape "$(POST "$field" "$cnt")")"
      #   ;;
      TEL)
         vcf="$(pdi_update_attrib "$vcf" TEL $cnt TYPE="$(POST teltype $cnt |grep -Exm1 'HOME|WORK|CELL|FAX')")"
         vcf="$(pdi_update_value "$vcf" "$field" "$cnt" "$(vcf_escape "$(POST "$field" "$cnt")")")"
         ;;
      *)
         vcf="$(pdi_update_value "$vcf" "$field" "$cnt" "$(vcf_escape "$(POST "$field" "$cnt")")")"
        ;;
    esac
done; done

# delete fields, first mark for deletion using delete_key
# this way the field enumeration is preserved during the process
# finally filter marked lines
delete_key="$(randomid)"
for delete in $(POST_KEYS |grep -xE '[A-Z][A-Z0-9-]*_delete_[0-9]+'); do
  f="${delete%%_*}"; c="${delete##*_}";
  [ "$(POST "$delete")" = "true" ] && vcf="$(pdi_update_value "$vcf" "$f" "$c" "delete=${delete_key}")"
done
vcf="$(printf '%s\n' "$vcf" |sed -E "/^[^:]+:delete=${delete_key}\$/d")"

if [ "$action" = addfield ]; then
  vcf="$(pdi_update_value "$vcf" "$newfield" $(( $(pdi_count "$vcf" "$newfield") + 1 )) '')"
fi
printf '%s' "$vcf" |grep -vx '' >"$tempfile"

case "$action" in
  addfield)
    REDIRECT "/cards/?o=${order}&f=${filter}&e=${card}"
    ;;
  update)
    if LOCK "$attfile"; then
      grep -F "	${card}" "$attfile" |while read course junk; do
        touch "$_DATA/ical/${course}"
      done
      sed -i -E "/^.+	${card}\$/d" "$attfile"
      seq 1 $(POST_COUNT attendance) |while read n; do
        printf '%s	%s\n' "$(POST attendance $n)" "$card"
      done >>"$attfile"
      grep -F "	${card}" "$attfile" |while read course junk; do
        touch "$_DATA/ical/${course}"
      done
      RELEASE "$attfile"
    else
      SET_COOKIE 0 message="COULD NOT UPDATE COURSE MAPPINGS"
    fi

    cp "$tempfile" "$cardfile"
    RELEASE_SLOCK "$cardfile"
    REDIRECT "/cards/?o=${order}&f=${filter}#${card}"
    ;;
  cancel)
    RELEASE_SLOCK "$cardfile"
    [ -f "$cardfile" ] \
    && REDIRECT "/cards/?o=${order}&f=${filter}#${card}" \
    || REDIRECT "/cards/?o=${order}&f=${filter}"
    ;;
  delete)
    rm "$cardfile"
    RELEASE_SLOCK "$cardfile"
    if LOCK "$attfile"; then
      grep -F "	${card}" "$attfile" |while read course junk; do
        touch "$_DATA/ical/${course}"
      done
      sed -i -E "/^.+	${card}\$/d" "$attfile"
      RELEASE "$attfile"
    else
      SET_COOKIE 0 message="COULD NOT UPDATE COURSE MAPPINGS"
    fi
    REDIRECT "/cards/?o=${order}&f=${filter}"
    ;;
esac
